/*
 * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.1 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
 * Copyright (c) 1998 Apple Computer, Inc.  All rights reserved. 
 *
 * IONetworkData.h
 *
 * HISTORY
 * 21-Apr-1999      Joe Liu (jliu) created.
 *
 */

#ifndef _IONETWORKDATA_H
#define _IONETWORKDATA_H

#define IONetworkParameter IONetworkData

/*! @enum IONetworkDataAccessFlags
    @discussion Flags to encode access attributes.
    @constant kIONetworkDataAccessTypeRead  read access type.
    @constant kIONetworkDataAccessTypeWrite write access type.
    @constant kIONetworkDataAccessTypeReset reset access type.
    @constant kIONetworkDataAccessTypeSerialize serialize access type. */

enum IONetworkDataAccessFlags {
    kIONetworkDataAccessTypeRead        = 0x01,
    kIONetworkDataAccessTypeWrite       = 0x02,
    kIONetworkDataAccessTypeReset       = 0x04,
    kIONetworkDataAccessTypeSerialize   = 0x08,
    kIONetworkDataAccessTypeMask        = 0xff,
};

/*! @define kIONetworkDataBasicAccessTypes
    @discussion The default access types supported by an IONetworkData
    object. Allow read() and serialize(). */

#define kIONetworkDataBasicAccessTypes \
       (kIONetworkDataAccessTypeRead | kIONetworkDataAccessTypeSerialize)

/*! @define kIONetworkDataImmutableAccessFlags
    @discussion Access flags that cannot be changed by setAccessFlags(),
    after the object has been initialized. */

#define kIONetworkDataImmutableAccessFlags   0

/*! @enum kIONetworkDataBufferType
    @discussion The type of buffer managed by the IONetworkData object.
    @constant kIONetworkDataBufferTypeInternal An internal data buffer
              allocated by the init() method.
    @constant kIONetworkDataBufferTypeExternal An external (persistent) data
              buffer.
    @constant kIONetworkDataBufferTypeNone No data buffer. The only useful 
              action perfomed by an IONetworkData object with this buffer type 
              is to call the access notification handler. */

enum IONetworkDataBufferType {
    kIONetworkDataBufferTypeInternal = 0,
    kIONetworkDataBufferTypeExternal,
    kIONetworkDataBufferTypeNone,
};

#ifdef KERNEL

#include <libkern/c++/OSSymbol.h>
#include <libkern/c++/OSSerialize.h>

/*! @class IONetworkData : public OSObject
    An IONetworkData manages a fixed-size named buffer.
    This class provides external access methods that can be called by
    an IOUserClient to allow user space access to the data buffer.
    In addition, serialization is supported, and therefore the object
    can be added to a property table to publish the data object.
    An unique name must be assigned to the object during initialization.
    An OSSymbol key will be created based on the assigned name, and this
    key should be used when the object is added to a dictionary.

    The level of access granted to the access methods can be restricted,
    by specifying a default set of access flags when the object is
    initialized, or modified later by calling setAccessFlags(). By
    default, each IONetworkData object created will support serialization,
    and will also allow its data buffer to be read through the read()
    access method.

    An access notification handler, in the form of a 'C' function, can
    be registered to receive a call each time the data buffer is accessed
    through an access method. Arguments provided to the handler will identify 
    the data object and the type of access that triggered the notification.
    The handler can therefore perform lazy update of the data buffer until
    an interested party tries to read or serialize the data. The notification
    handler can also take over the default action performed by the access
    methods when the buffer type is set to kIONetworkDataBufferTypeNone.
    This will prevent the access methods from accessing the data buffer,
    and allow the handler to override the access protocol.

    This object is used by IONetworkInterface to export statistics
    to user space. */


class IONetworkData : public OSObject
{
    OSDeclareDefaultStructors(IONetworkData)

public:

/*! @typedef Action
    Defines the prototype of the access notification handler that is
    called when an access method is called.
    @param target The target of the notification.
    @param data The IONetworkData object being accessed, or the
           sender of the notification.
    @param accessType A bit will be set indicating a particular
           access type.
    @param arg An action argument.
    @param buffer Pointer to the accessor's buffer. Valid for
           read() and write() accesses.
    @param bufSize Pointer to the size of the accessor's buffer.
    @param offset An offset from the start of the data buffer to begin
           reading or writing. */

    typedef IOReturn (*Action)(OSObject *      target,
                               IONetworkData * data,
                               UInt32          accessType,
                               void *          arg,
                               void *          buffer,
                               UInt *          bufSize,
                               UInt            offset);

protected:
    const OSSymbol *          _key;       // key associated with this object.
    UInt32                    _access;    // access flags.
    void *                    _buffer;    // Data buffer.
    IONetworkDataBufferType   _bufType;   // buffer type
    UInt                      _size;      // data buffer size.
    OSObject *                _tapTarget; // target for access notification.
    Action                    _tapAction; // the function to call.
    void *                    _tapArg;    // arbitrary notification argument.

/*! @function free
    @abstract Free the IONetworkData instance. */

    virtual void free();

public:

/*! @function initialize
    @abstract IONetworkData class initializer. */

    static void initialize();

/*! @function withInternalBuffer
    @abstract Factory method that will construct and initialize an
    IONetworkData instance with an internal buffer.
    @param name A name assigned to this object.
    @param size Number of bytes to allocate for the internal buffer.
    @param accessFlags The initial access flags.
    @param target Notification target object. The target will be notified
           when this instance receives a call to one of its access methods.
    @param action The notification action.
    @param arg An argument to pass to the notification action.
    @result An IONetworkData instance on success, or 0 otherwise. */

    static IONetworkData *
           withInternalBuffer(const char * name,
                              UInt         size,
                              UInt32       accessFlags = 
                                           kIONetworkDataBasicAccessTypes,
                              OSObject *   target = 0,
                              Action       action = 0,
                              void *       arg    = 0);

/*! @function withExternalBuffer
    @abstract Factory method that will construct and initialize an
    IONetworkData instance with an external buffer.
    @param name A name assigned to this object.
    @param size Size of the external data buffer.
    @param extBuffer Pointer to the external buffer.
    @param accessFlags The initial access flags.
    @param target Notification target object. The target will be notified
           when this instance receives a call to one of its access methods.
    @param action The notification action.
    @param arg An argument to pass to the notification action.
    @result An IONetworkData instance on success, or 0 otherwise. */

    static IONetworkData *
           withExternalBuffer(const char * name,
                              UInt         size,
                              void *       extBuffer,
                              UInt32       accessFlags =
                                           kIONetworkDataBasicAccessTypes,
                              OSObject *   target = 0,
                              Action       action = 0,
                              void *       arg    = 0);

/*! @function withNoBuffer
    @abstract Factory method that will construct and initialize an
    IONetworkData instance with no data buffer. The notification handler
    must intervene when the IONetworkData is accessed.
    @param name A name assigned to this object.
    @param size Size of the phantom data buffer.
    @param accessFlags The initial access flags.
    @param target Notification target object. The target will be notified
           when this instance receives a call to one of its access methods.
    @param action The notification action.
    @param arg An argument to pass to the notification action.
    @result An IONetworkData instance on success, or 0 otherwise. */

    static IONetworkData * withNoBuffer(const char * name,
                                        UInt         size,
                                        UInt32       accessFlags,
                                        OSObject *   target,
                                        Action       action,
                                        void *       arg = 0);

/*! @function init
    @abstract Initialize an IONetworkData instance.
    @param name A name assigned to this object.
    @param bufferType The type of buffer associated with this instance.
    @param size Size of the data buffer.
    @param extBuffer Pointer to an external data buffer.
    @param accessFlags The initial access flags.
           Can be later modified by calling setAccessFlags().
    @param target Notification target object. The target will be notified
           when this instance receives a call to one of its access methods.
    @param action The notification action.
    @param arg An argument to pass to the notification action.
    @result true if initialized successfully, false otherwise. */

    virtual bool init(const char *            name,
                      IONetworkDataBufferType bufferType,
                      UInt                    size,
                      void *                  extBuffer   = 0,
                      UInt32                  accessFlags =
                                              kIONetworkDataBasicAccessTypes,
                      OSObject *              target      = 0,
                      Action                  action      = 0,
                      void *                  arg         = 0);

/*! @function setAccessFlags
    @abstract Change the access flags.
    @param accessFlags A bitfield of access flag bits indicating the
           supported access types. See IONetworkDataAccessFlags enum. */

    virtual void setAccessFlags(UInt32 accessFlags);

/*! @function setNotificationTarget
    @abstract Register a target/action to handle access notification.
    @discussion A notification is sent by an IONetworkData to the registered
    notification handler when the data object is accessed through an access
    method.
    @param target The target object that implements the notification action.
           If 0, then notification is disabled.
    @param action The notification action.
           If 0, then notification is disabled.
    @param arg An optional argument passed to the notification handler. */

    virtual void setNotificationTarget(OSObject * target,
                                       Action     action,
                                       void *     arg = 0);

/*! @function getBuffer
    @result A pointer to the data buffer. Returns 0 if the buffer type is
            kIONetworkDataBufferTypeNone. */

    virtual const void *     getBuffer() const;

/*! @function getBufferType
    @result The type of buffer managed by this instance.
            See IONetworkDataBufferType enum definition. */

    virtual IONetworkDataBufferType  getBufferType() const;

/*! @function getAccessFlags
    @result The access flags. */

    virtual UInt32           getAccessFlags() const;

/*! @function getTarget
    @result The notification target. */

    virtual OSObject *       getTarget() const;

/*! @function getAction
    @result The notification action. */

    virtual Action           getAction() const;

/*! @function getArgument
    @result The optional notification argument. */

    virtual void *           getArgument() const;

/*! @function getKey
    @abstract Get an OSSymbol key associated with this instance.
    @discussion During initialization, IONetworkData will create an
    OSSymbol key based on its assigned name.
    @result An OSSymbol key generated from the assigned name. */

    virtual const OSSymbol * getKey() const;

/*! @function getSize
    @result The size of the data buffer managed by this instance in bytes. */

    virtual UInt getSize() const;

/*! @function updateBuffer
    @abstract Update the data buffer from a source buffer provided by the
    caller.
    @param bytes Pointer to a source buffer provided by the caller.
    @param inLength Length of the source buffer, or the number of bytes
           to copy to the data buffer.
    @param offset A byte offset from the start of the destination data buffer
           to begin writing.
    @result true on success, false otherwise. */

    virtual bool updateBuffer(const void * inBuf, UInt length, UInt offset = 0);

/*! @function copyBuffer
    @abstract Copy the data buffer to a destination buffer provided by the 
    caller.
    @param outBuf Pointer to the destination buffer.
    @param inOutSize Pointer to an integer containing the size of the
    destination buffer before the call. And is overwritten by this method to
    hold the actual number of bytes copied to the destination buffer.
    @param offset A byte offset from the start of the source data buffer
           to begin reading.
    @result true on success, false otherwise. */

    virtual bool copyBuffer(void * outBuf, UInt * inOutLength, UInt offset = 0) 
    const;

/*! @function clearBuffer
    @abstract Clear the entire data buffer by filling it with zeroes.
    @result true if the buffer was cleared, false otherwise. */

    virtual bool clearBuffer();

/*! @function reset
    @abstract An access method to reset the data buffer.
    @discussion Handle an external request to reset the data buffer.
    If notication is enabled, then the notification handler is called
    after the data buffer has been cleared.
    @result kIOReturnSuccess on success,
            kIOReturnNotWritable if reset access is not permitted,
            or an error from the notification handler. */

    virtual IOReturn reset();

/*! @function read
    @abstract An access method to read from the data buffer.
    @discussion Handle an external request to read from the data buffer
    and copy it to the destination buffer provided by the accessor.
    If notification is enabled, then the notification handler is called
    before the data buffer is copied to the destination buffer. The 
    notification handler may use this opportunity to intervene and
    to update the contents of the data buffer.
    @param outBuffer Pointer to the destination buffer.
    @param inOutSize Pointer to an integer containing the size of the
    destination buffer before the call. And is overwritten by this method to
    hold the actual number of bytes copied to the destination buffer.
    @param offset An offset from the start of the source data buffer to begin
    reading.
    @result kIOReturnSuccess on success,
            kIOReturnBadArgument if one of the arguments provided is invalid,
            kIOReturnNotReadable if read access is not permitted,
            or an error from the notification handler. */

    virtual IOReturn read(void * outBuffer, UInt * inOutSize, UInt offset = 0);

/*! @function write
    @abstract An access method to write to the data buffer.
    @discussion Handle an external request to write to the data buffer
    using a source buffer provided by the accessor. After checking that
    the data object supports write accesses, the data buffer is updated
    if it exists. Then the registered notification handler is called.
    @param inBuffer Pointer to the source buffer.
    @param size The number of bytes to write to the data buffer.
    @param offset An offset from the start of the destination data buffer to
    begin writing.
    @result kIOReturnSuccess on success,
            kIOReturnBadArgument if one of the arguments provided is invalid,
            kIOReturnNotWritable if write access is not permitted,
            or an error from the notification handler. */

    virtual IOReturn write(void * inBuffer, UInt size, UInt offset = 0);

/*! @function serialize
    @abstract Serialize the object, including the data buffer if it
    exists.
    @discussion If notification is enabled, then the notification
    handler is called just before the data buffer is serialized.
    @param s An OSSerialize object.
    @result true on success, false otherwise. */

    virtual bool serialize(OSSerialize * s) const;
};

#endif /* KERNEL */

#endif /* !_IONETWORKDATA_H */
